/* 
 *  pam_auth.c - Check user through PAM (Pluggable Authentication Module).
 *  NOT SUPPORTED IN CURRENT VERSION !!! Mail mc@tyumen.ru if you want to use this.
 *
 *  Copyright (c) 1998-1999  Maxim Chirkov <mc@skyway.ru>  
 */

#include "getstatd.h"

static char *PAM_username = "";
static char *PAM_password = "";

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#ifdef USE_PAM

static int pam_conv (int num_msg, const struct pam_message **msg, struct pam_response **resp, void *appdata_ptr){
    int count = 0, replies = 0;
    struct pam_response *reply = NULL;
    int size = 0;

    for (count = 0; count < num_msg; count++)
      {
	size += sizeof (struct pam_response);
	reply = realloc (reply, size); /* ANSI: is malloc() if reply==NULL */
	if (!reply)
	    return PAM_CONV_ERR;

	switch (msg[count]->msg_style)
	  {
	case PAM_PROMPT_ECHO_ON:
	    reply[replies].resp_retcode = PAM_SUCCESS;
	    if (PAM_username != NULL){
		reply[replies++].resp = strdup(PAM_username); /* never NULL */
	    } else {
		reply[replies++].resp = strdup("");
	    }
	    break;

	case PAM_PROMPT_ECHO_OFF:
	    reply[replies].resp_retcode = PAM_SUCCESS;
	    if (PAM_password != NULL){
		reply[replies++].resp = strdup(PAM_password); /* never NULL */
	    } else {
		reply[replies++].resp = strdup("");
	    }

	    break;

	case PAM_TEXT_INFO:
	    reply[replies].resp_retcode = PAM_SUCCESS;
	    reply[replies++].resp = NULL;
	    break;

	case PAM_ERROR_MSG:
	default:
	    free (reply);
	    return PAM_CONV_ERR;
	  }
      }

    if (resp)
        *resp = reply;
    else
        free (reply);

    return PAM_SUCCESS;
}
#endif

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */


int check_pam_pwd (char * user_name, char * user_passwd){

#ifdef USE_PAM

    struct pam_conv pam_conversation;
    pam_handle_t *pamh;
    int pam_error;
/*
 * Fill the pam_conversion structure
 */
    if (use_pam != 1){
        return AUTH_DENY;
    }
    bzero((char *) &pam_conversation, sizeof (struct pam_conv));
    pam_conversation.conv = &pam_conv;

    pam_error = pam_start ("getstatd", user_name, &pam_conversation, &pamh);

    if (pam_error != PAM_SUCCESS) {
	log_info("Can't start PAM");
        return AUTH_DENY;
    }

    PAM_username = user_name;
    PAM_password = user_passwd;
/*
 * Validate the user
 */
    pam_error = pam_authenticate (pamh, PAM_SILENT);
    if (pam_error == PAM_SUCCESS) {
	return AUTH_ALLOW;
    } else {
        return AUTH_DENY;
    }
    pam_end (pamh, pam_error);

#else
    return AUTH_DENY;
#endif

}